package sample;

import java.util.HashMap;
import java.util.Hashtable;
import java.util.Map;

import junit.framework.TestCase;

public class ComponentTestCase extends TestCase
{

	public ComponentTestCase(String name)
	{
		super(name);
	}

	public static void setMock(String className, String methodName, Object returnValue)
	{
		testData.put(makeKey(className, methodName), returnValue);
	}

	public static void setMock(String className, String methodName)
	{
		setMock(className, methodName, new Object());
	}

	public static Object getMockReturnValue(String className, String methodName)
	{
		return testData.get(makeKey(className, methodName));
	}

	public void assertCalled(String className, String methodName)
	{
		if (!isCalled(className, methodName))
			fail(
				"The method '" + methodName + "' in class '" + className + "' was expected to be called but it wasn't");
	}

	public Object getArgument(String className, String methodName, String argumentName)
	{
		Object argument = null;
		Hashtable arguments = (Hashtable)callsMade.get(makeKey(className, methodName));
		if (arguments != null)
			argument = arguments.get(argumentName);
		return argument;
	}

	public static void indicateCalled(String className, String methodName, Hashtable arguments)
	{
		callsMade.put(makeKey(className, methodName), arguments);
	}

	public static boolean isCalled(String className, String methodName)
	{
		return callsMade.get(makeKey(className, methodName)) != null;
	}

	public void assertArgumentPassed(String className, String methodName, String argumentName, Object argumentValue)
	{
		Object argument = getArgument(className, methodName, argumentName);
		if (argument == null || !argument.equals(argumentValue))
			fail(
				"The argument '"
					+ argumentName
					+ "' of method '"
					+ methodName
					+ "' in class '"
					+ className
					+ " ' should have the value '"
					+ argumentValue
					+ "' but it was '"
					+ argument
					+ "'!");
	}

	private static String makeKey(String className, String methodName)
	{
		return className + "." + methodName;
	}

	private static Hashtable testData = new Hashtable();
	private static Hashtable callsMade = new Hashtable();

	private static Map exceptions = new HashMap();
	
	public static void setMockException(String className, String methodName, Throwable t)
	{
		exceptions.put(makeKey(className, methodName), t);
	}

	public static Throwable getMockException(String className, String methodName)
	{
		return (Throwable)exceptions.get(makeKey(className, methodName));
	}

}
